if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_TASK_CLOSE')
	DROP PROCEDURE @SCHEMA@.LSW_TASK_CLOSE
GO	
CREATE PROCEDURE @SCHEMA@.LSW_TASK_CLOSE(@userId DECIMAL, @taskId DECIMAL, @returnStatus DECIMAL OUTPUT) AS

DECLARE @taskUserId DECIMAL
DECLARE @taskGroupId DECIMAL
DECLARE @taskStatus VARCHAR(2)
DECLARE @taskProcessRef DECIMAL
DECLARE @taskExecutionStatus DECIMAL

DECLARE @flgCanClose CHAR

SET @returnStatus = -1

SELECT @taskUserId = USER_ID,@taskGroupId = GROUP_ID,@taskStatus = STATUS,@taskProcessRef = START_PROCESS_REF,@taskExecutionStatus = EXECUTION_STATUS
	FROM @SCHEMA@.LSW_TASK
	WHERE TASK_ID = @taskId
IF (@@ROWCOUNT <= 0)
  BEGIN
  PRINT 'Invalid TASK_ID ' + CONVERT(VARCHAR, @taskId) + '. No data found'
  RETURN
  END

/* See if we can close this task */
SET @flgCanClose = 'F'
IF (SUBSTRING(@taskStatus,1,1) <> '3')
  BEGIN
  IF (@taskProcessRef IS NULL)
    SET @flgCanClose = 'T'
  ELSE
    IF ((@taskExecutionStatus = 1) OR (@taskExecutionStatus = 4)) SET @flgCanClose = 'T'

  /* See if we own this task */
  IF (@userId <> @taskUserId)
    BEGIN
    SET @flgCanClose = 'F';
    PRINT 'Not the owner of task ' + CONVERT(VARCHAR, @taskId)
    END
  END

/* OK, so now close this task */
IF (@flgCanClose = 'T')
  BEGIN
  BEGIN TRANSACTION
  UPDATE @SCHEMA@.LSW_TASK SET STATUS = '32', CLOSE_DATETIME = GETUTCDATE(), CLOSE_BY = @userId, EXECUTION_STATUS = 1
  		 WHERE TASK_ID = @taskId;
  COMMIT TRANSACTION;
  SET @returnStatus = 0;
  END

GO
if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_TASK_DELETE')
	DROP PROCEDURE @SCHEMA@.LSW_TASK_DELETE
GO
CREATE PROCEDURE @SCHEMA@.LSW_TASK_DELETE(@userId DECIMAL, @taskId DECIMAL, @returnStatus DECIMAL OUTPUT) AS

DECLARE @taskUserId DECIMAL
DECLARE @taskGroupId DECIMAL
DECLARE @taskStatus VARCHAR(2)
DECLARE @taskProcessRef DECIMAL
DECLARE @taskExecutionStatus DECIMAL

DECLARE @flgCanDelete CHAR

SET @returnStatus = -1

SELECT @taskUserId = USER_ID,@taskGroupId = GROUP_ID,@taskStatus = STATUS,@taskProcessRef = START_PROCESS_REF,@taskExecutionStatus = EXECUTION_STATUS
	FROM @SCHEMA@.LSW_TASK
	WHERE TASK_ID = @taskId
IF (@@ROWCOUNT <= 0)
  BEGIN
  PRINT 'Invalid TASK_ID ' + CONVERT(VARCHAR, @taskId) + '. No data found'
  RETURN
  END

/* See if we can delete this task */
SET @flgCanDelete = 'F'
IF ((SUBSTRING(@taskStatus,1,1) = '2') OR (SUBSTRING(@taskStatus,1,1) = '3'))
  BEGIN
  IF (@taskProcessRef IS NULL)
    SET @flgCanDelete = 'T'
  ELSE
    IF ((@taskExecutionStatus = 1) OR (@taskExecutionStatus = 4)) SET @flgCanDelete = 'T'

  /* See if we own this task */
  IF (@userId <> @taskUserId)
    BEGIN
    SET @flgCanDelete = 'F';
    PRINT 'Not the owner of task ' + CONVERT(VARCHAR, @taskId)
    END
  END

/* OK, so now delete this task */
IF (@flgCanDelete = 'T')
  BEGIN
  BEGIN TRANSACTION
  IF (SUBSTRING(@taskStatus,1,1) = '2') 
    UPDATE @SCHEMA@.LSW_TASK SET STATUS = '92'
  		 WHERE TASK_ID = @taskId;
  ELSE
    UPDATE @SCHEMA@.LSW_TASK SET STATUS = '91'
  		 WHERE TASK_ID = @taskId;

  COMMIT TRANSACTION;
  SET @returnStatus = 0;
  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_HOUSE_KEEPING')
	DROP PROCEDURE @SCHEMA@.LSW_HOUSE_KEEPING
GO
CREATE PROCEDURE @SCHEMA@.LSW_HOUSE_KEEPING(@modeHK DECIMAL) AS

DECLARE @rootTaskId DECIMAL
DECLARE @taskId DECIMAL
DECLARE @taskStatus VARCHAR(2)
DECLARE @flgDelete CHAR(1)
DECLARE @statuses VARCHAR(64)
DECLARE @sql NVARCHAR(128)
DECLARE @sqlParams NVARCHAR(128)
DECLARE @flgStatusNotExist CHAR(1)

IF (@modeHK = 1)
  BEGIN
  /* Delete orphaned and DELETED task attachments*/
  BEGIN TRANSACTION
  DELETE FROM @SCHEMA@.LSW_TASK_FILE WHERE TASK_ID IN (SELECT TASK_ID FROM @SCHEMA@.LSW_TASK WHERE STATUS IN (91,92))
  DELETE FROM @SCHEMA@.LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM @SCHEMA@.LSW_TASK_FILE) OR FILE_ID NOT IN (SELECT FILE_ID FROM @SCHEMA@.LSW_TASK,@SCHEMA@.LSW_TASK_FILE WHERE @SCHEMA@.LSW_TASK.TASK_ID = @SCHEMA@.LSW_TASK_FILE.TASK_ID)
  DELETE FROM @SCHEMA@.LSW_TASK_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM @SCHEMA@.LSW_FILE) OR TASK_ID NOT IN (SELECT @SCHEMA@.LSW_TASK.TASK_ID FROM @SCHEMA@.LSW_TASK,@SCHEMA@.LSW_TASK_FILE WHERE @SCHEMA@.LSW_TASK.TASK_ID = @SCHEMA@.LSW_TASK_FILE.TASK_ID)

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM @SCHEMA@.LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM @SCHEMA@.LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM @SCHEMA@.LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
  DELETE FROM @SCHEMA@.LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM @SCHEMA@.LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM @SCHEMA@.LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)

  COMMIT TRANSACTION
  END
ELSE IF (@modeHK > 1)
  BEGIN

  /* Define what task statuses we are interested in */
  SET @statuses = ''
  IF (@modeHK >= 2) SET @statuses = '91,92'
  IF (@modeHK >= 3) SET @statuses = @statuses + ',21'
  IF (@modeHK >= 4) SET @statuses = @statuses + ',31,32'

  CREATE TABLE #TEMP_DELETE_TASK (
  TASK_ID DECIMAL(12,0) NOT NULL
  )

  /* Go through each root task id */
  DECLARE curAllRootTasks CURSOR SCROLL FOR SELECT DISTINCT ORIG_TASK_ID FROM @SCHEMA@.LSW_TASK
  OPEN curAllRootTasks

  WHILE (1 = 1)
    BEGIN
    FETCH curAllRootTasks INTO @rootTaskId
    IF (@@FETCH_STATUS = -1) BREAK;

    /* Check each task belonging to the current root task group. If ALL tasks in this group belong to
        the statuses we are interested in, then we can remove these tasks */
    EXEC('DECLARE curRootTask CURSOR SCROLL FOR SELECT TASK_ID, STATUS FROM @SCHEMA@.LSW_TASK WHERE ORIG_TASK_ID = ' + @rootTaskId)
    OPEN curRootTask

    SET @flgDelete = 'T'
    WHILE (1 = 1)
      BEGIN
      FETCH curRootTask INTO @taskId,@taskStatus
      IF (@@FETCH_STATUS = -1) BREAK;

      SET @flgStatusNotExist = 'F'
      SET @sql = 'SELECT @pflgStatusNotExist = ''T'' WHERE ' + @taskStatus + ' NOT IN (' + @statuses + ')'
      SET @sqlParams = '@pflgStatusNotExist CHAR OUTPUT'
      EXEC sp_executesql @sql, @sqlParams, @pflgStatusNotExist = @flgStatusNotExist OUTPUT

      IF (@flgStatusNotExist = 'T')
        BEGIN
        SET @flgDelete = 'F'
        BREAK
        END

      INSERT INTO #TEMP_DELETE_TASK VALUES (@taskId)

      END

    CLOSE curRootTask
    DEALLOCATE curRootTask

    IF (@flgDelete = 'T')
      BEGIN
      DECLARE curDeleteTask CURSOR SCROLL FOR SELECT TASK_ID FROM #TEMP_DELETE_TASK
      OPEN curDeleteTask

      WHILE (1 = 1)
        BEGIN
        FETCH curDeleteTask INTO @taskId
        IF (@@FETCH_STATUS = -1) BREAK;
        BEGIN TRANSACTION
        DELETE FROM @SCHEMA@.LSW_TASK_NARR WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK_FILE WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK_ADDR WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK_IPF_DATA WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK_EXTACT_DATA WHERE TASK_ID = @taskId
        DELETE FROM @SCHEMA@.LSW_TASK WHERE TASK_ID = @taskId
        COMMIT TRANSACTION
        END

      CLOSE curDeleteTask
      DEALLOCATE curDeleteTask
      END

      DELETE FROM #TEMP_DELETE_TASK

    END

  CLOSE curAllRootTasks
  DEALLOCATE curAllRootTasks

  DROP TABLE #TEMP_DELETE_TASK

  /* Delete all attachment files that are not referenced by any task */
  DELETE FROM @SCHEMA@.LSW_FILE WHERE FILE_ID NOT IN (SELECT FILE_ID FROM @SCHEMA@.LSW_TASK_FILE)

  /* Delete orphaned task-scoped temporary groups */
  DELETE FROM @SCHEMA@.LSW_USR_GRP_MEM_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM @SCHEMA@.LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM @SCHEMA@.LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
  DELETE FROM @SCHEMA@.LSW_USR_GRP_XREF WHERE GROUP_ID IN (
    SELECT g.GROUP_ID FROM @SCHEMA@.LSW_USR_GRP_XREF g WHERE NOT EXISTS (SELECT 1 FROM @SCHEMA@.LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) AND g.GROUP_TYPE = 2)
    
  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_ERASE_TEMP_GROUPS')
	DROP PROCEDURE @SCHEMA@.LSW_ERASE_TEMP_GROUPS
GO
CREATE PROCEDURE @SCHEMA@.LSW_ERASE_TEMP_GROUPS AS

  /* This procedure deletes all temporary groups that are no longer referenced by a task. */

  BEGIN

    DECLARE @groupId DECIMAL

    DECLARE groupsToDelete SCROLL CURSOR FOR
      SELECT DISTINCT g.GROUP_ID 
      FROM @SCHEMA@.LSW_USR_GRP_XREF g 
      WHERE NOT EXISTS (SELECT 1 FROM @SCHEMA@.LSW_TASK t WHERE g.GROUP_ID = ABS(t.GROUP_ID)) 
      AND g.GROUP_TYPE = 2
      FOR READ ONLY

    BEGIN TRANSACTION

      OPEN groupsToDelete

      FETCH groupsToDelete INTO @groupId

      WHILE @@FETCH_STATUS = 0
        BEGIN
          
          DELETE FROM @SCHEMA@.LSW_USR_GRP_MEM_XREF WHERE GROUP_ID = @groupId
          DELETE FROM @SCHEMA@.LSW_USR_GRP_XREF WHERE GROUP_ID = @groupId
          
          FETCH groupsToDelete INTO @groupId
          
        END

      CLOSE groupsToDelete

      DEALLOCATE groupsToDelete

    COMMIT

  END
GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_ERASE_BPD_INST_NO_GROUP')
	DROP PROCEDURE @SCHEMA@.LSW_ERASE_BPD_INST_NO_GROUP
GO
CREATE PROCEDURE @SCHEMA@.LSW_ERASE_BPD_INST_NO_GROUP @bpdInstanceId DECIMAL AS

  /* This procedure deletes a BPD instance and its dependent entries. It does NOT delete temporary groups. */

  BEGIN
  
    DELETE FROM @SCHEMA@.LSW_BPD_INSTANCE_DOC_PROPS WHERE DOC_ID IN (SELECT DOC_ID FROM @SCHEMA@.LSW_BPD_INSTANCE_DOCUMENTS WHERE BPD_INSTANCE_ID = @bpdInstanceId)

    DELETE FROM @SCHEMA@.LSW_BPD_INSTANCE_DOCUMENTS WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_BPD_INSTANCE_VARIABLES WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_BPD_INSTANCE_DATA WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_BPD_NOTIFICATION WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_RUNTIME_ERROR WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_BPD_INSTANCE WHERE BPD_INSTANCE_ID = @bpdInstanceId

    DELETE FROM @SCHEMA@.LSW_INST_MSG_INCL WHERE BPD_INSTANCE_ID = @bpdInstanceId 

    DELETE FROM @SCHEMA@.LSW_INST_MSG_EXCL WHERE BPD_INSTANCE_ID = @bpdInstanceId

  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_ERASE_BPD_INSTANCE')
	DROP PROCEDURE @SCHEMA@.LSW_ERASE_BPD_INSTANCE
GO
CREATE PROCEDURE @SCHEMA@.LSW_ERASE_BPD_INSTANCE @bpdInstanceId DECIMAL AS

  /* This procedure deletes a BPD instance and its dependent entries, including temporary groups. */

  BEGIN
  
    /* Delete the BPD instance itself */
    EXECUTE @SCHEMA@.LSW_ERASE_BPD_INST_NO_GROUP @bpdInstanceId

    /* Delete orphaned task-scoped temporary groups */
    EXECUTE @SCHEMA@.LSW_ERASE_TEMP_GROUPS

  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_ERASE_TASK_NO_GROUP')
	DROP PROCEDURE @SCHEMA@.LSW_ERASE_TASK_NO_GROUP
GO
CREATE PROCEDURE @SCHEMA@.LSW_ERASE_TASK_NO_GROUP @taskId DECIMAL AS

  /* This procedure deletes a task and its dependent entries. It does NOT delete temporary groups that were created for this task. */

  BEGIN

    DELETE FROM @SCHEMA@.LSW_TASK_ADDR WHERE TASK_ID = @taskId

    DELETE FROM @SCHEMA@.LSW_TASK_EXECUTION_CONTEXT WHERE TASK_ID = @taskId

    DELETE FROM @SCHEMA@.LSW_TASK_NARR WHERE TASK_ID = @taskId

    DELETE FROM @SCHEMA@.LSW_TASK_FILE WHERE TASK_ID = @taskId

    DELETE FROM @SCHEMA@.LSW_TASK_IPF_DATA WHERE TASK_ID = @taskId
    
    DELETE FROM @SCHEMA@.LSW_TASK_EXTACT_DATA WHERE TASK_ID = @taskId
    
    DELETE FROM @SCHEMA@.LSW_TASK WHERE TASK_ID = @taskId
    
  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_ERASE_TASK')
	DROP PROCEDURE @SCHEMA@.LSW_ERASE_TASK
GO
CREATE PROCEDURE @SCHEMA@.LSW_ERASE_TASK @taskId DECIMAL AS

  /* This procedure deletes a task and its dependent entries, including temporary groups that were created for this task. */

  BEGIN

    /* Delete the task itself */
    EXECUTE @SCHEMA@.LSW_ERASE_TASK_NO_GROUP @taskId

    /* Cleanup: Delete task-scoped temporary group */
    EXECUTE @SCHEMA@.LSW_ERASE_TEMP_GROUPS
    
  END

GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_BPD_INST_DELETE_NO_GROUP')
	DROP PROCEDURE @SCHEMA@.LSW_BPD_INST_DELETE_NO_GROUP
GO
CREATE PROCEDURE @SCHEMA@.LSW_BPD_INST_DELETE_NO_GROUP @bpdInstanceId DECIMAL AS

  /* This procedure deletes a BPD instance and its dependent entries, as well as tasks that are part of this instance. It does NOT delete temporary groups. */

  BEGIN

    DECLARE @taskId DECIMAL

    DECLARE tasksToDelete SCROLL CURSOR FOR
      SELECT TASK_ID
      FROM @SCHEMA@.LSW_TASK
      WHERE BPD_INSTANCE_ID = @bpdInstanceId
      FOR READ ONLY

    BEGIN TRANSACTION

      OPEN tasksToDelete

      FETCH tasksToDelete INTO @taskId

      WHILE @@FETCH_STATUS = 0
        BEGIN
          EXECUTE @SCHEMA@.LSW_ERASE_TASK_NO_GROUP @taskId
          FETCH tasksToDelete INTO @taskId
        END

      CLOSE tasksToDelete

      DEALLOCATE tasksToDelete

      EXECUTE @SCHEMA@.LSW_ERASE_BPD_INST_NO_GROUP @bpdInstanceId

    COMMIT

  END
GO

if exists (SELECT * FROM INFORMATION_SCHEMA.ROUTINES WHERE  ROUTINE_NAME = 'LSW_BPD_INSTANCE_DELETE')
	DROP PROCEDURE @SCHEMA@.LSW_BPD_INSTANCE_DELETE
GO
CREATE PROCEDURE @SCHEMA@.LSW_BPD_INSTANCE_DELETE @bpdInstanceId DECIMAL AS

  /* This procedure deletes a BPD instance and its dependent entries, as well as tasks that are part of this instance and temporary groups. */

  BEGIN
  
    /* Delete the BPD instance itself */
    EXECUTE @SCHEMA@.LSW_BPD_INST_DELETE_NO_GROUP @bpdInstanceId
    
    /* Cleanup: Delete orphaned task-scoped temporary groups */
    EXECUTE @SCHEMA@.LSW_ERASE_TEMP_GROUPS

  END
GO
