#
# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# (C) Copyright IBM Corporation 2013. All Rights Reserved.
# This sample program is provided AS IS and may be used, executed, copied and
# modified without royalty payment by customer (a) for its own instruction and study,
# (b) in order to develop applications designed to run with an IBM WebSphere product,
# either for customer's own internal use or for redistribution by customer, as part of
# such an application, in customer's own products.
#
import AdminUtilities
import getopt
import sys

def usage ():
    print('Usage: Use this script to get/modify the configured security properties.\n')
    print('       -E|--de DE_name -option')
    print('                       -g|--get property_name')
    print('                       -s|--set property_name , new_value')
    print('                       -a|--add console_property_name , constraint_value')
    print('                               |action_policy_name , role to be added')
    print('                       -r|--remove console_property_name , constraint_value')
    print('                               |action_policy_name , role to be removed')
    sys.exit(2)
    
# This function returns the name and type of a Deployment Environment.
# The de type is used to determine if the DE in question is from an Express edition of BPM in which case, the config objects are slightly different.

def getDE(de):
    deIds = AdminUtilities.convertToList(AdminConfig.getid ('/Cell:/BPMCellConfigExtension:/BPMDeploymentEnvironment:/'))
    for i in range(len(deIds)):
        if (AdminConfig.showAttribute(deIds[i], 'name') == de):
            deName = AdminConfig.showAttribute(deIds[i], 'name')
            deType = AdminConfig.showAttribute(deIds[i],'environmentType')
            return deName,deType
    print "No DE with the given name exists."
    usage()
    
# In the case of BPM Express or Single De's, the cluster is assumed to have all the required capabilities.   
# But for multiple DE's, each cluster maybe set up with different capabilities and the properties defined for a cluster are based on these capabilities.
# Property "viewUser"  - capability needed : Support
# All other properties - capability needed : Application
# This function returns the right cluster for each property.
    
def getClusterOrServer(deId, type, capability):
    if type == "Express":
        containmentPath = "/Cell:/BPMCellConfigExtension:/BPMDeploymentEnvironment:%s/BPMServer:/" % deId
    else:
        containmentPath = "/Cell:/BPMCellConfigExtension:/BPMDeploymentEnvironment:%s/BPMCluster:/" % deId
    clusterOrServerIds = AdminUtilities.convertToList(AdminConfig.getid(containmentPath))
    for i in range(len(clusterOrServerIds)):
                    capabilities = AdminUtilities.convertToList(AdminConfig.showAttribute(clusterOrServerIds[i], 'capabilities'))
                    for j in range(len(capabilities)):
                        list =  capabilities[j].split(";")
                        for k in range(len(list)): 
                            if (list[k] == capability):
                                clusterOrServerName = AdminConfig.showAttribute(clusterOrServerIds[i], 'name')
                                return clusterOrServerName
    print "No cluster with the required capabilities exists."
    usage()
    
#Returns the current value for a property

def get(deId, type, property):
    print "\nCurrent value for property", property , "in DE", deId , "is:"   
    DEContainmentPath = "/Cell:/BPMCellConfigExtension:/BPMDeploymentEnvironment:%s/" % deId 
    clusterOrServerName = getClusterOrServer(deId, type, "Application")
    clusterPathPS = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMProcessServer:/" % clusterOrServerName
    clusterPathPC = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMProcessCenter:/" % clusterOrServerName
    serverPath = "/Server:%s/BPMServerConfigExtension:/BPMProcessCenter:/" % clusterOrServerName
                
# BPMServerSecurity
    if property in ("externalUserQueryLimit" , "deploySnapshotUsingHttps" , "securityNameTransformer" , "wildcardProcessingOptimized"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/" % clusterPathPC    
            print AdminConfig.showAttribute(AdminConfig.getid(objectContainmentPath) ,property)              

# BPMServerSecurityUsers
    elif property in ("notifyError" , "userToCloseTask" , "userToCreateTask"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % clusterPathPC   
            print AdminConfig.showAttribute(AdminConfig.getid(objectContainmentPath) ,property)  
                
# BPMServerSecurityGroups                
    elif property in ("bpmAdminGroup" , "bpmAuthorGroup" , "processCenterInstall" , 
		              "offlineInstall" , "collaborationAdmin" , "showXmlMetadata" , 
		              "debug" , "processHelpAccess"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % clusterPathPC   
            print AdminConfig.showAttribute(AdminConfig.getid(objectContainmentPath) ,property)          

# LDAPOptions 
    elif property in ("twUserNameAttribute" , "twUserDescriptionAttribute" , "twGroupNameAttribute" , "twGroupDescriptionAttribute"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % clusterPathPC
            ldapOptions = AdminUtilities.convertToList(AdminConfig.getid(objectContainmentPath))
            for i in range(len(ldapOptions)):
                    ldapOption = AdminConfig.showAttribute((ldapOptions[i]), 'name')
                    if (property == ldapOption):
                        print AdminConfig.showAttribute(ldapOptions[i], 'value')
                        break                        
            
# BPMVirtualHostInfo              
    elif property in ("hostname" , "port" , "transportProtocol"):
            if type == "Express":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % clusterPathPC
            print AdminConfig.showAttribute(AdminConfig.getid(objectContainmentPath) ,property)     
                                               
# BPMActionPolicy
    elif property in ("ACTION_ABORT_INSTANCE" , "ACTION_SUSPEND_INSTANCE" , "ACTION_RESUME_INSTANCE" , "ACTION_ADD_COMMENT" , "ACTION_ADD_HELP_REQUEST" , 
                      "ACTION_RESPOND_HELP_REQUEST" , "ACTION_ASSIGN_TASK" , "ACTION_ASSIGN_AND_RUN_TASK" , "ACTION_REASSIGN_TASK" , "ACTION_REASSIGN_TASK_USER_ROLE" ,
                      "ACTION_CHANGE_TASK_DUEDATE" , "ACTION_CHANGE_INSTANCE_DUEDATE" , "ACTION_CHANGE_TASK_PRIORITY" , "ACTION_MOVE_TOKEN" , "ACTION_DELETE_TOKEN" , 
                      "ACTION_INJECT_TOKEN" , "ACTION_VIEW_PROCESS_DIAGRAM" , "ACTION_VIEW_PROCESS_AUDIT" , "ACTION_CHANGE_CRITICAL_PATH" , "ACTION_ADD_DOCUMENT" ,
                      "ACTION_UPDATE_DOCUMENT" , "ACTION_DELETE_DOCUMENT" , "ACTION_DELETE_INSTANCE" , "ACTION_FIRE_TIMER" , "ACTION_RETRY_INSTANCE" , "ACTION_SEND_EVENT"):
            if type == "Express":
                objectContainmentPath = "/Server:%s/BPMServerConfigExtension:/BPMPortal:/BPMActionPolicy:/BPMPolicyAction:/" % clusterOrServerName
            else:
                objectContainmentPath = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMPortal:/BPMActionPolicy:/BPMPolicyAction:/" % clusterOrServerName
            policyActions = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))                
            for i in range(len(policyActions)):
                    actionType = AdminConfig.showAttribute(policyActions[i], 'type')
                    if(property == actionType):
                        print AdminConfig.showAttribute(policyActions[i], 'roles')     
                        
# BPMConsoleSection
    elif property in ("console.blackout.periods" , "console.bulk.user.attribute.assignment" , "console.em.jms.error.queue" , "console.group.management" , 
                      "console.instrumentation" , "console.manage.caches" , "console.manage.epvs" , "console.monitor" , 
                      "console.process.monitor" , "console.synchronous.queues" , "console.user.management" , "console.user.synchronization" , 
                      "console.task.cleanup"):
            objectContainmentPath = "/BPMConsoleElement:/"
            consoleElements = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))
            for i in range(len(consoleElements)):
                    elementName = AdminConfig.showAttribute((consoleElements[i]), 'name')
                    if (property == elementName):
                        constraintIds = AdminConfig.showAttribute((consoleElements[i]), 'constraints')
                        constraints = constraintIds.replace("[", "")
                        constraints = constraints.replace("]", "")
                        constraints = constraints.split(" ")
                        for j in range(len(constraints)):
                            constraints[j] = "\"" + constraints[j] + "\""
                            print "constraint" , j , ":" , AdminConfig.showAttribute(constraints[j], 'value') 
                        break    
                        
# BPMAuthAliasRoleType
    elif property in ("BPCUser" , "BPMAuthor" , "BPMUser" , "BPMWebserviceUser" , "CEIDbUser" , "CEIUser" , "CellAdmin" , 
                      "DeAdmin" , "EmbeddedECMTechnicalUser" , "EventManagerUser" , "PerformanceDWUser" , 
                      "ProcessCenterUser" , "ProcessServerUser" , "RALUser" , "SCAUser" , "SCADeploymentUser"):
                objectContainmentPath = "%sBPMAuthAliasRoleType:/" % DEContainmentPath
                authAliasRoleTypes = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))
                for i in range(len(authAliasRoleTypes)):
                    authAliasRoleType = AdminConfig.showAttribute(authAliasRoleTypes[i], 'name')
                    if (property == authAliasRoleType):
                        authAliasName = AdminConfig.showAttribute((authAliasRoleTypes[i]), 'authAliasName')
                        print "authAliasName : " , authAliasName 
                        break 
                        
# BPMPerformanceDataWarehouse
    clusterOrServerName = getClusterOrServer(deId, type, "Support")
    if (property == "viewUser"):
            if type == "Express":
                objectContainmentPath = "/Server:%s/BPMServerConfigExtension:/BPMPerformanceDataWarehouse:/" % clusterOrServerName
            else:
                objectContainmentPath = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMPerformanceDataWarehouse:/BPMViewManager:/BPMSystem:/" % clusterOrServerName
            print AdminConfig.showAttribute(AdminConfig.getid(objectContainmentPath) ,property)   
                

def set(deId, type, property,value):
    get(deId, type, property)
    DEContainmentPath = "/Cell:/BPMCellConfigExtension:/BPMDeploymentEnvironment:%s/" % deId 
    clusterOrServerName = getClusterOrServer(deId, type, "Application")
    clusterPathPS = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMProcessServer:/" % clusterOrServerName
    clusterPathPC = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMProcessCenter:/" % clusterOrServerName
    serverPath = "/Server:%s/BPMServerConfigExtension:/BPMProcessCenter:/" % clusterOrServerName
   
# All cases where the Cluster capability is Application - All properties except viewUser:
    clusterOrServerName = getClusterOrServer(deId, type, "Application")     
# BPMServerSecurity
    if property in ("externalUserQueryLimit" , "deploySnapshotUsingHttps" , "securityNameTransformer" , "wildcardProcessingOptimized"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/" % clusterPathPC   
            AdminConfig.modify(AdminConfig.getid(objectContainmentPath),[[property, value]])
# BPMServerSecurityUsers
    elif property in ("notifyError" , "userToCloseTask" , "userToCreateTask"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityUsers:/" % clusterPathPC   
            AdminConfig.modify(AdminConfig.getid(objectContainmentPath),[[property, value]])
            		        
# BPMServerSecurityGroups
    elif property in ("bpmAdminGroup" , "bpmAuthorGroup" , "processCenterInstall" , 
		              "offlineInstall" , "collaborationAdmin" , "showXmlMetadata" , 
		              "debug" , "processHelpAccess"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMServerSecurityGroups:/" % clusterPathPC  
            AdminConfig.modify(AdminConfig.getid(objectContainmentPath),[[property, value]])
            
# LDAPOptions
    elif property in ("twUserNameAttribute" , "twUserDescriptionAttribute" , "twGroupNameAttribute" , "twGroupDescriptionAttribute"):
            if type == "Express":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMServerSecurity:/BPMLdapOption:/" % clusterPathPC  
            ldapOptions = AdminUtilities.convertToList(AdminConfig.getid(objectContainmentPath))
            for i in range(len(ldapOptions)):
                    ldapOption = AdminConfig.showAttribute((ldapOptions[i]), 'name')
                    if (property == ldapOption):
                        AdminConfig.modify(ldapOptions[i], [['value', []]])        
                        AdminConfig.modify(ldapOptions[i], [['value', value]]) 
                          
# BPMAuthAliasRoleType
    elif property in ("BPCUser" , "BPMAuthor" , "BPMUser" , "BPMWebserviceUser" , "CEIDbUser" , "CEIUser" , "CellAdmin" , 
                      "DeAdmin" , "EmbeddedECMTechnicalUser" , "EventManagerUser" , "PerformanceDWUser" , 
                      "ProcessCenterUser" , "ProcessServerUser" , "RALUser" , "SCAUser" , "SCADeploymentUser"):
            objectContainmentPath = "%sBPMAuthAliasRoleType:/" % DEContainmentPath
            authAliasRoleTypes = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))
            for i in range(len(authAliasRoleTypes)):
                    authAliasRoleType = AdminConfig.showAttribute(authAliasRoleTypes[i], 'name')
                    if (property == authAliasRoleType):
                        AdminConfig.modify(authAliasRoleTypes[i], [['authAliasName', []]])
                        AdminConfig.modify(authAliasRoleTypes[i], [['authAliasName', value]])
                        break 
                     
# BPMVirtualHostInfo          
    elif property in ("hostname" , "port" , "transportProtocol"):
            if type == "Express":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % serverPath
            elif type == "Process Server":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % clusterPathPS
            elif type == "Process Center":
                objectContainmentPath = "%sBPMVirtualHostInfo:/" % clusterPathPC  
            AdminConfig.modify(AdminConfig.getid(objectContainmentPath) ,[[property,value]]) 
   
# BPMActionPolicy
    elif property in ("ACTION_ABORT_INSTANCE" , "ACTION_SUSPEND_INSTANCE" , "ACTION_RESUME_INSTANCE" , "ACTION_ADD_COMMENT" , "ACTION_ADD_HELP_REQUEST" , 
                      "ACTION_RESPOND_HELP_REQUEST" , "ACTION_ASSIGN_TASK" , "ACTION_ASSIGN_AND_RUN_TASK" , "ACTION_REASSIGN_TASK" , "ACTION_REASSIGN_TASK_USER_ROLE" ,
                      "ACTION_CHANGE_TASK_DUEDATE" , "ACTION_CHANGE_INSTANCE_DUEDATE" , "ACTION_CHANGE_TASK_PRIORITY" , "ACTION_MOVE_TOKEN" , "ACTION_DELETE_TOKEN" , 
                      "ACTION_INJECT_TOKEN" , "ACTION_VIEW_PROCESS_DIAGRAM" , "ACTION_VIEW_PROCESS_AUDIT" , "ACTION_CHANGE_CRITICAL_PATH" , "ACTION_ADD_DOCUMENT" ,
                      "ACTION_UPDATE_DOCUMENT" , "ACTION_DELETE_DOCUMENT" , "ACTION_DELETE_INSTANCE" , "ACTION_FIRE_TIMER" , "ACTION_RETRY_INSTANCE" , "ACTION_SEND_EVENT"):
            print "\nThis property supports only the addition/removal of roles. Check usage."
            usage()
            
# BPMConsoleSection
    elif property in ("console.blackout.periods" , "console.bulk.user.attribute.assignment" , "console.em.jms.error.queue" , "console.group.management" , 
                      "console.instrumentation" , "console.manage.caches" , "console.manage.epvs" , "console.monitor" , 
                      "console.process.monitor" , "console.synchronous.queues" , "console.user.management" , "console.user.synchronization" , 
                      "console.task.cleanup"):
            print "\nThis property supports only the addition/removal of constraints. Check usage."
            usage()
            
# All cases where the Cluster capability is Application - Only viewUser:
# BPMPerformanceDataWarehouse                
    clusterOrServerName = getClusterOrServer(deId, type, "Support")
    if (property == "viewUser"):
            if type == "Express":
                objectContainmentPath = "/Server:%s/BPMServerConfigExtension:/BPMPerformanceDataWarehouse:/" % clusterOrServerName
            else:
                objectContainmentPath = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMPerformanceDataWarehouse:/BPMViewManager:/BPMSystem:/" % clusterOrServerName
            AdminConfig.modify(AdminConfig.getid(objectContainmentPath), [[property,value]]) 

    print "\nThe given value for the property was set successfully."
    AdminConfig.save()
    get(deId, type, property)
    
def modifyConstraintsOrActions(deID, type, property, value, action):
# BPMConsoleSection
    get(deID, type, property)
    if property in ("console.blackout.periods" , "console.bulk.user.attribute.assignment" , "console.em.jms.error.queue" , "console.group.management" , "console.instrumentation" , 
                    "console.manage.caches" , "console.manage.epvs" , "console.monitor" , "console.process.monitor" , "console.synchronous.queues" , 
                    "console.user.management" , "console.user.synchronization" , "console.task.cleanup"):
                objectContainmentPath = "/BPMConsoleElement:/"
                consoleElements = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))
                for i in range(len(consoleElements)):
                    elementName = AdminConfig.showAttribute((consoleElements[i]), 'name')
                    if (property == elementName):
                        constraintIds = AdminConfig.showAttribute((consoleElements[i]), 'constraints')
                        constraints = constraintIds.replace("[", "")
                        constraints = constraints.replace("]", "")
                        constraints = constraints.split(" ")
                        if (action == "remove"):
                            flag = None
                            for  j in range(len(constraints)):
                                constraints[j] = "\"" + constraints[j] + "\""
                                if (value == AdminConfig.showAttribute(constraints[j], 'value')): 
                                    AdminConfig.remove(constraints[j])  
                                    print "\nThe given constraint was removed successfully."
                                    flag = 1
                                    break
                            if not flag:
                                    print "\nConstraint not defined. No changes have been made to the configuration."
                                    break
                        if (action == "add"):
                            flag = None
                            for  j in range(len(constraints)):
                                constraints[j] = "\"" + constraints[j] + "\""
                                if (value == AdminConfig.showAttribute(constraints[j], 'value')): 
                                    flag = 1
                                    break
                            if flag:
                                print "\nConstraint already defined. No changes have been made to the configuration." 
                                break                            
                            if not flag:
                                AdminConfig.create('BPMConsoleConstraint', consoleElements[i], [['type' , 'role'],['value', value]])
                                print "\nThe given value for the property was set successfully."
                                break
                        break        
# BPMActionPolicy
    elif property in ("ACTION_ABORT_INSTANCE" , "ACTION_SUSPEND_INSTANCE" , "ACTION_RESUME_INSTANCE" , "ACTION_ADD_COMMENT" , "ACTION_ADD_HELP_REQUEST" , 
                      "ACTION_RESPOND_HELP_REQUEST" , "ACTION_ASSIGN_TASK" , "ACTION_ASSIGN_AND_RUN_TASK" , "ACTION_REASSIGN_TASK" , "ACTION_REASSIGN_TASK_USER_ROLE" ,
                      "ACTION_CHANGE_TASK_DUEDATE" , "ACTION_CHANGE_INSTANCE_DUEDATE" , "ACTION_CHANGE_TASK_PRIORITY" , "ACTION_MOVE_TOKEN" , "ACTION_DELETE_TOKEN" , 
                      "ACTION_INJECT_TOKEN" , "ACTION_VIEW_PROCESS_DIAGRAM" , "ACTION_VIEW_PROCESS_AUDIT" , "ACTION_CHANGE_CRITICAL_PATH" , "ACTION_ADD_DOCUMENT" ,
                      "ACTION_UPDATE_DOCUMENT" , "ACTION_DELETE_DOCUMENT" , "ACTION_DELETE_INSTANCE" , "ACTION_FIRE_TIMER" , "ACTION_RETRY_INSTANCE" , "ACTION_SEND_EVENT"):
        clusterOrServerName = getClusterOrServer(deID, type, "Application")
        if type == "Express":
                objectContainmentPath = "/Server:%s/BPMServerConfigExtension:/BPMPortal:/BPMActionPolicy:/BPMPolicyAction:/" % clusterOrServerName
        else:
                objectContainmentPath = "/ServerCluster:%s/BPMClusterConfigExtension:/BPMPortal:/BPMActionPolicy:/BPMPolicyAction:/" % clusterOrServerName
        policyActions = AdminUtilities.convertToList(AdminConfig.getid (objectContainmentPath))   
        for i in range(len(policyActions)):
                actionType = AdminConfig.showAttribute(policyActions[i], 'type')
                if(property == actionType):
                    if(action == "add"):
                        AdminConfig.modify(policyActions[i], [['roles', value]])
                        break
                    elif(action == "remove"):  
                        roles = AdminConfig.showAttribute(policyActions[i], 'roles').split(";")
                        AdminConfig.modify(policyActions[i], [['roles', []]])
                        for j in range(len(roles)):
                            if(roles[j]!=value):
                                AdminConfig.modify(policyActions[i], [['roles', roles[j]]])
        print "\nThe operation was completed successfully."   
                                          
    AdminConfig.save()                       
    get(deID, type, property)   

def main():
    try:
        option, property = getopt.getopt(sys.argv[0:], 'E:g:s:a:r:u', ["De", "get=", "set=", "add=" , "remove=" , "usage"])
    except getopt.GetoptError:
        usage()
        print(err)
    for option,args in option:
        if option in ("-E", "--de"):
            de,deType = getDE(args)
        elif option in ("-g", "--get"):
            get(de, deType, args)
        elif option in ("-s", "--set"):
            args = args.split(',') 
            if len(args) != 2:
                print "\nINFO: Wrong number of arguments"
                usage()
            set(de, deType, args[0], args[1])
        elif option in ("-a", "--add"):
            args = args.split(',') 
            if len(args) != 2:
                print "\nINFO: Wrong number of arguments"
                usage()
            modifyConstraintsOrActions(de, deType, args[0], args[1] , "add")
        elif option in ( "-r" , "--remove"):
            args = args.split(',') 
            if len(args) != 2:
                print "\nINFO: Wrong number of arguments"
                usage()
            modifyConstraintsOrActions(de, deType, args[0], args[1], "remove")
        elif option in ("-u", "--usage"):
            usage()
        else:
            usage()
            sys.exit()

if __name__ == "__main__":
    main()